
-- This is a VHDL template file generated from ghdlsim.xml
-- using perio.xsl
-- Changes to this file WILL BE LOST. Edit the source file.
--
-- (c) 2012-2013, Martin Strubel // hackfin@section5.ch
--
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library work;
use work.ghdlsim.all;

entity decode_tap_registers is
	port (
		ce        : in  std_logic;
		ctrl      : out tap_registers_WritePort;
		stat      : in  tap_registers_ReadPort;
		data_in   : in  std_logic_vector(32-1 downto 0);
		data_out  : out std_logic_vector(32-1 downto 0);
		addr      : in  std_logic_vector(BV_MMR_CFG_tap_registers);
		we        : in  std_logic;
		clk       : in  std_logic
	);
end entity decode_tap_registers;

architecture behaviour of decode_tap_registers is

	constant ADDR_MSB : natural := 7;
	subtype REG_SIZE1B is integer range 7 downto 0;
	subtype REG_SIZE2B is integer range 15 downto 0;
	subtype REG_SIZE4B is integer range 31 downto 0;

	-- Default register size:
	subtype reg_size1_t is std_logic_vector(7 downto 0);
	subtype reg_size2_t is std_logic_vector(15 downto 0);
	subtype reg_size4_t is std_logic_vector(31 downto 0);

	signal uaddr : unsigned(BV_MMR_CFG_tap_registers);

	--------------------------
	-- Register allocations --
	
	-- Register TAP_EMUCTRL
	signal bit_emureq : std_logic := '0';
	signal reg_select_reg : std_logic_vector(BV_SELECT_REG) := "000";
	signal bit_emuexec : std_logic := '0';
	signal bit_core_reset : std_logic := '0';

	-- Register TAP_EMUIR
	signal reg_tap_emuir : reg_size4_t;
		
	-- Register TAP_SIMCTRL
	signal bit_sim_throttle : std_logic := '1';

	-- Register SIM_SLEEPCYCLES
	signal reg_sim_sleepcycles : reg_size2_t := x"fa00";
		

begin

	uaddr    <= unsigned(addr);

-- Registermap TAP

ioproc_tap_registers:
	process(clk)
		variable addr : unsigned(ADDR_MSB downto 0);
		constant ZEROPAD   :
		unsigned(ADDR_MSB - uaddr'length downto 0) :=
			(others => '0');
	begin
		if rising_edge(clk) then

		if ce = '1' then
			addr := ZEROPAD & uaddr;
			-- WRITE
			if we = '1' then
				case addr is
				when R_TAP_EMUCTRL =>
					bit_emureq <= data_in(B_EMUREQ);
					reg_select_reg <= data_in(BV_SELECT_REG);
					bit_emuexec <= data_in(B_EMUEXEC);
					bit_core_reset <= data_in(B_CORE_RESET);
				when R_TAP_EMUIR =>
					reg_tap_emuir <=  data_in(REG_SIZE4B);
				when R_TAP_SIMCTRL =>
					bit_sim_throttle <= data_in(B_SIM_THROTTLE);
				when R_SIM_SLEEPCYCLES =>
					reg_sim_sleepcycles <=  data_in(REG_SIZE2B);

				when others =>
				end case;
			-- READ
			else
				data_out <= (others => '0'); -- XXX default 0
				case addr is
				when R_TAP_IDCODE =>
					data_out(REG_SIZE4B) <= stat.tap_idcode;
				when R_TAP_EMUDATA =>
					data_out(REG_SIZE4B) <= stat.tap_emudata;
				when R_TAP_EMUSTAT =>
					data_out(B_EMUACK) <= stat.emuack;
					data_out(B_EMURDY) <= stat.emurdy;
					data_out(BV_CORE_SPEC) <= stat.core_spec;
				when R_TAP_EMUPC =>
					data_out(REG_SIZE4B) <= stat.tap_emupc;
				when R_TAP_EMUCTRL =>
					data_out(B_EMUREQ) <= bit_emureq;
					data_out(BV_SELECT_REG) <= reg_select_reg;
					data_out(B_EMUEXEC) <= bit_emuexec;
					data_out(B_CORE_RESET) <= bit_core_reset;
				when R_TAP_EMUIR =>
					data_out(REG_SIZE4B) <= reg_TAP_EMUIR;
				when R_TAP_SIMCTRL =>
					data_out(B_SIM_THROTTLE) <= bit_sim_throttle;
				when R_SIM_SLEEPCYCLES =>
					data_out(REG_SIZE2B) <= reg_SIM_SLEEPCYCLES;

				when others =>
				end case;
			end if;
		end if;
		end if;
	end process;

	-- Assignments
	ctrl.emureq <= bit_emureq;
	ctrl.select_reg <= reg_select_reg;
	ctrl.emuexec <= bit_emuexec;
	ctrl.core_reset <= bit_core_reset;
	ctrl.tap_emuir <= reg_tap_emuir;
	ctrl.sim_throttle <= bit_sim_throttle;
	ctrl.sim_sleepcycles <= reg_sim_sleepcycles;

end behaviour;
