--! \file libnetpp.vhdl Generated autowrapper library package
--
-- This file is generated from "libnetpp.chdl"
-- Modifications will be lost, please edit "libnetpp.chdl"
--
--
-- (c) 2011-2013 Martin Strubel <hackfin@section5.ch>

library ieee;
 use ieee.std_logic_1164.all;
 use ieee.numeric_std.all; -- Unsigned

--! \brief GHPI netpp wrapper package
--! Provides a few functions to access netpp master functionality from
--! the simulation.


package ghpi_netpp is

 --! \defgroup GlobalSignals Global Signals
 --! These are experimental signals

 --! \addtogroup GlobalSignals
 --! \{

 ----- GLOBAL SIGNALS -----

 --! Global throttle signal. If high, use as throttle signal
 --! to virtual FIFO implementations such as VirtualFX2Fifo.
 signal global_throttle : std_logic := '1';

 --! Global number of wait cycles for some entities to sleep.
 signal global_waitcycles : natural := 50000;

 --! Global debugger clock in MHz
 signal global_dbgclk : std_logic := '0';

 --! \}

 ----- NETPP SLAVE API -----

 --! \addtogroup GHPI_Netpp
 --! \{

 --! Initializes the netpp slave structures for a simulation
 function netpp_init(name : string)
  return integer;

 ----- NETPP MASTER API -----
 subtype token_t is integer;
 subtype netpphandle_t is integer;
 type handle_t is access integer;

    -- The function wrapper appends a \000 character
 --! Opens a netpp target somewhere on the network
 function device_open(id: string)
  return netpphandle_t;
 --! Retrieves the netpp token by name
 function device_gettoken(h: netpphandle_t; id: string)
  return token_t;

 --! \}

 --! \addtogroup VirtualRAM
 --! \{

 ----- RAM API -----
 subtype ram16_t is unsigned(15 downto 0);
 subtype byte_t is unsigned(7 downto 0);
 subtype regaddr_t is unsigned(7 downto 0);
 type rambuf_t is access integer;

 --! Allocates a new virtual RAM
 --! @param size Size in words (not bytes)
 --! @param name A unique name, used for netpp export
 function ram_new(name : string; size : integer) return rambuf_t;

 --! \}

 --! \addtogroup VFifoAPI
 --! \{

 ----- FIFO API -----

 subtype fifoflag_t is unsigned(0 to 5);
 -- FIFO flag indices:
 constant RX : natural := 0; --! in: Read advance, out: FIFO not empty
 constant TX : natural := 1; --! in: Write advance, out: FIFO not full
 constant RXE : natural := 2; --! out: LOW when FIFO almost not empty
 constant TXF : natural := 3; --! out: LOW when FIFO almost full
 constant OVR : natural := 4; --! out: overrun bit. Write 1 to clear.
 constant UNR : natural := 5; --! out: underrun bit. W1C.

 -- FLAG assignments for FX2 emulation
 constant TX_PROG : natural := 0;
 constant TX_EMPTY : natural := 1;
 constant RX_FULL : natural := 2;


 --! Access type. These do not directly map into C struct pointers,
 --! see functions using duplexfifo_t_ghdl as arguments for example
 --! on how to access the handle in C.
 type duplexfifo_t is access integer;

 --! Allocates a new virtual FIFO
 --! @param name A unique name, used for netpp export
 --! @param size Size in words (not bytes)
 --! @param wordsize Word size (1 or 2)
 function fifo_new(name : string; size : integer; wordsize : integer)
  return duplexfifo_t;

 --! \}

 --! \addtogroup VBusAPI
 --! \{
 ----- NETPP BUS API -----

 type bus_t is access integer;
 --! Bus flags. 0: read, 1: write, 2: read ready
 subtype busflag_t is unsigned(0 to 2);

 --! Create a new virtual bus.
 --! @param name Name in the netpp root hierarchy
 --! @param wordsize Data word width in bytes
 function bus_new(name : string; wordsize : integer)
  return bus_t;

 --! \}


 ----- NETPP REMOTE FRAMEBUFFER API -----

 --! \addtogroup VFramebuf
 --! \{

 subtype framebuffer_t is integer; --! Framebuffer handle
 subtype pixel_t is unsigned(15 downto 0); --! Pixel type
 type pixarray_t is array(natural range <>) of pixel_t; --! Pixel array

 constant VIDEOMODE_8BIT : natural := 1; --! 8 Bit grayscale
 constant VIDEOMODE_UYVY : natural := 3; --! UYVY 16 bpp mode
 constant VIDEOMODE_INDEXED : natural := 17; --! Indexed 16bit mode

 --! \}
function device_open_wrapped(id : string) return netpphandle_t;
 attribute foreign of device_open_wrapped :
  function is "VHPIDIRECT sim_device_open_wrapped";
--! Set integer value on netpp remote device
--! @param t The property token, obtained by device_gettoken()
--! @param v A 32 bit integer
function device_set_int(h : netpphandle_t; t : token_t; v : integer) return integer;
 attribute foreign of device_set_int :
  function is "VHPIDIRECT sim_device_set_int";
--! Set register value on netpp remote device
function device_set_register(h : netpphandle_t; t : token_t; v : integer) return integer;
 attribute foreign of device_set_register :
  function is "VHPIDIRECT sim_device_set_register";
function device_gettoken_wrapped(h : netpphandle_t; id : string) return token_t;
 attribute foreign of device_gettoken_wrapped :
  function is "VHPIDIRECT sim_device_gettoken_wrapped";
--! Close connection to remote device
procedure device_close(h : netpphandle_t);
 attribute foreign of device_close :
  procedure is "VHPIDIRECT sim_device_close";
--! Read from dummy register map. At the moment only 8 bit wide
--! @deprecated Use the VirtualBus API instead
--! @param addr Register map address
--! @param data Register map data
procedure regmap_read(addr : regaddr_t; data : out unsigned);
 attribute foreign of regmap_read :
  procedure is "VHPIDIRECT sim_regmap_read";
--! Write to dummy register map. At the moment only 8 bit wide
--! @deprecated Use the VirtualBus API instead
--! @param addr Register map address
--! @param data Register map data
procedure regmap_write(addr : regaddr_t; data : unsigned);
 attribute foreign of regmap_write :
  procedure is "VHPIDIRECT sim_regmap_write";
--! Sleep for 'cycles' microseconds
--! This is a true CPU sleep, simulation will pause for this period.
--! @param cycles sleep time in us
procedure usleep(cycles : integer);
 attribute foreign of usleep :
  procedure is "VHPIDIRECT sim_usleep";
--! Throttle simulation
--! @param activity When toggled, do not sleep
--! @param cycles sleep time in us
procedure throttle(activity : byte_t; cycles : integer);
 attribute foreign of throttle :
  procedure is "VHPIDIRECT sim_throttle";
--! \addtogroup VFramebuf
--! \{
--!
-- DELIMITER --
--! Initialize remote frame buffer device
--! @param dev A netpp framebuffer capable device handle
--! @param w Width of the framebuffer
--! @param h Height of the frame buffer
--! @param buftype One of #VIDEOMODE_8BIT, #VIDEOMODE_UYVY, #VIDEOMODE_INDEXED
--! For supported video modes, see display/videomodes.h
function initfb(dev : netpphandle_t; w : integer; h : integer; buftype : integer) return framebuffer_t;
 attribute foreign of initfb :
  function is "VHPIDIRECT sim_initfb";
--! Set pixel on remote frame buffer
--! @param x X coordinate
--! @param y Y coordinate
--! @param pixel Pixel value
procedure setpixel(fb : framebuffer_t; x : integer; y : integer; pixel : pixel_t);
 attribute foreign of setpixel :
  procedure is "VHPIDIRECT sim_setpixel";
--! Write entire remote frame buffer
--! @param data Pointer to VHDL frame buffer type
procedure setfb(fb : framebuffer_t; data : pixarray_t);
 attribute foreign of setfb :
  procedure is "VHPIDIRECT sim_setfb";
--! Send update event to framebuffer
procedure updatefb(fb : framebuffer_t);
 attribute foreign of updatefb :
  procedure is "VHPIDIRECT sim_updatefb";
--! Release remote framebuffer
procedure releasefb(fb : framebuffer_t);
 attribute foreign of releasefb :
  procedure is "VHPIDIRECT sim_releasefb";
--! \}
--! \addtogroup VFifoAPI
--! \{
--!
-- DELIMITER --
function fifo_new_wrapped(name : string; size : integer; wordsize : integer) return duplexfifo_t;
 attribute foreign of fifo_new_wrapped :
  function is "VHPIDIRECT sim_fifo_new_wrapped";
--! The FIFO I/O routine.
--! @param data Pointer to data being written, if TX flag set,
--! Data is modified with the 'read' FIFO data when RX
--! flag set. If FIFO is not ready for READ or WRITE
--! operation (empty or full), an underrun respective
--! overrun condition will occur and be signalled in the
--! flags(OVR) and flags(UNR) bits. Clearing this error
--! condition is achieved by setting these error flags to
--! '1'.
--! The word size of the data bit vector is defined by
--! the wsize argument to fifo_thread_init()
--! @param flags The FIFO control (in) and status (out) flags.
--! When calling this function the first time, you should
--! check the status by setting all flags to '0'.
--! On return, the (RX) and (TX) fill state can be read
--! from the RXE and TXF flags (for burst reading) and
--! from the RX and TX flags (current, absolute FIFO fill
--! state. On subsequent calls, the RX and TX flags
--! indicate, which action (Read word/Write word) should
--! be taken. The current status is always returned in the
--! flags array.
procedure fifo_rxtx(df : inout duplexfifo_t; data : inout unsigned; flags : inout fifoflag_t);
 attribute foreign of fifo_rxtx :
  procedure is "VHPIDIRECT sim_fifo_rxtx";
--! Delete FIFO
procedure fifo_del(fifo : inout duplexfifo_t);
 attribute foreign of fifo_del :
  procedure is "VHPIDIRECT sim_fifo_del";
--! \}
function bus_new_wrapped(name : string; width : integer) return bus_t;
 attribute foreign of bus_new_wrapped :
  function is "VHPIDIRECT sim_bus_new_wrapped";
--! \addtogroup VBusAPI
--! \{
--!
-- DELIMITER --
--! Virtual Bus I/O
--! @param vbus Bus handle
--! @param addr Address bus
--! @param data Data bus
--! @param flags Bus flags
procedure bus_rxtx(vbus : inout bus_t; addr : inout unsigned; data : inout unsigned; flags : inout busflag_t);
 attribute foreign of bus_rxtx :
  procedure is "VHPIDIRECT sim_bus_rxtx";
--! Delete Bus
procedure bus_del(vbus : inout bus_t);
 attribute foreign of bus_del :
  procedure is "VHPIDIRECT sim_bus_del";
--! \}
function ram_new_wrapped(name : string; size : integer) return rambuf_t;
 attribute foreign of ram_new_wrapped :
  function is "VHPIDIRECT sim_ram_new_wrapped";
--! \addtogroup VirtualRAM
--! \{
--!
-- DELIMITER --
--! Write to RAM buffer
--! @param addr word address
--! @param data input data
procedure ram_write(ram : inout rambuf_t; addr : inout unsigned; data : out ram16_t);
 attribute foreign of ram_write :
  procedure is "VHPIDIRECT sim_ram_write";
--! Read from RAM buffer
--! @param addr word address
--! @param data output data
procedure ram_read(ram : inout rambuf_t; addr : inout unsigned; data : ram16_t);
 attribute foreign of ram_read :
  procedure is "VHPIDIRECT sim_ram_read";
--! Delete and free RAM buffer
procedure ram_del(ram : inout rambuf_t);
 attribute foreign of ram_del :
  procedure is "VHPIDIRECT sim_ram_del";
--! \}
function netpp_init_wrapped(name : string) return integer;
 attribute foreign of netpp_init_wrapped :
  function is "VHPIDIRECT sim_netpp_init_wrapped";
function get_ptr(dev : netpphandle_t) return handle_t;
 attribute foreign of get_ptr :
  function is "VHPIDIRECT sim_get_ptr";
procedure set_ptr(h : inout handle_t);
 attribute foreign of set_ptr :
  procedure is "VHPIDIRECT sim_set_ptr";
end package;
package body ghpi_netpp is
 function netpp_init(name : string)
  return integer is
 begin
  return netpp_init_wrapped(name & NUL);
 end function;
 function device_open(id: string)
  return netpphandle_t is
  variable dev : netpphandle_t;
    begin
  dev := device_open_wrapped(id & NUL);
  if dev < 0 then
   assert false report "Failed to open netpp remote device"
    severity failure;
  end if;
  return dev;
 end device_open;
 function device_gettoken(h: netpphandle_t; id: string)
  return token_t is
    begin
  return device_gettoken_wrapped(h, id & NUL);
 end device_gettoken;
 -- Wrap functions with string parameter: Just append a null termination
 -- to become C standard compatible.
 function ram_new(name : string; size : integer) return rambuf_t is
 begin
  return ram_new_wrapped(name & NUL, size);
 end function;
 function fifo_new(name : string; size : integer; wordsize : integer )
  return duplexfifo_t is
 begin
  return fifo_new_wrapped(name & NUL, size, wordsize);
 end function;
 function bus_new(name : string; wordsize : integer )
  return bus_t is
 begin
  return bus_new_wrapped(name & NUL, wordsize);
 end function;
function device_open_wrapped(id : string) return netpphandle_t is
begin
 assert false report "VHPI" severity failure;
end function;
function device_set_int(h : netpphandle_t; t : token_t; v : integer) return integer is
begin
 assert false report "VHPI" severity failure;
end function;
function device_set_register(h : netpphandle_t; t : token_t; v : integer) return integer is
begin
 assert false report "VHPI" severity failure;
end function;
function device_gettoken_wrapped(h : netpphandle_t; id : string) return token_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure device_close(h : netpphandle_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure regmap_read(addr : regaddr_t; data : out unsigned) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure regmap_write(addr : regaddr_t; data : unsigned) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure usleep(cycles : integer) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure throttle(activity : byte_t; cycles : integer) is
begin
 assert false report "VHPI" severity failure;
end procedure;
function initfb(dev : netpphandle_t; w : integer; h : integer; buftype : integer) return framebuffer_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure setpixel(fb : framebuffer_t; x : integer; y : integer; pixel : pixel_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure setfb(fb : framebuffer_t; data : pixarray_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure updatefb(fb : framebuffer_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure releasefb(fb : framebuffer_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
function fifo_new_wrapped(name : string; size : integer; wordsize : integer) return duplexfifo_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure fifo_rxtx(df : inout duplexfifo_t; data : inout unsigned; flags : inout fifoflag_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure fifo_del(fifo : inout duplexfifo_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
function bus_new_wrapped(name : string; width : integer) return bus_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure bus_rxtx(vbus : inout bus_t; addr : inout unsigned; data : inout unsigned; flags : inout busflag_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure bus_del(vbus : inout bus_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
function ram_new_wrapped(name : string; size : integer) return rambuf_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure ram_write(ram : inout rambuf_t; addr : inout unsigned; data : out ram16_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure ram_read(ram : inout rambuf_t; addr : inout unsigned; data : ram16_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
procedure ram_del(ram : inout rambuf_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
function netpp_init_wrapped(name : string) return integer is
begin
 assert false report "VHPI" severity failure;
end function;
function get_ptr(dev : netpphandle_t) return handle_t is
begin
 assert false report "VHPI" severity failure;
end function;
procedure set_ptr(h : inout handle_t) is
begin
 assert false report "VHPI" severity failure;
end procedure;
end package body;
