--
-- This is a VHDL package file generated from ghdlsim.xml
-- using vhdlregs.xsl
-- Changes to this file WILL BE LOST. Edit the source file.
--
-- Implements a 'msb+1' bit address wide register map as VHDL package
--
-- Set the msb by specifying the --param msb `number` option to xsltproc
--
-- (c) 2007-2014, Martin Strubel // hackfin@section5.ch
--
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work; use work.iomap_config.all;

package ghdlsim is
	subtype  regaddr_t is unsigned(7 downto 0);

	subtype  REG_SIZE1B is integer range 7 downto 0;
	subtype  REG_SIZE2B is integer range 15 downto 0;
	subtype  REG_SIZE3B is integer range 23 downto 0;
	subtype  REG_SIZE4B is integer range 31 downto 0;



-------------------------------------------------------------------------
-- Address segment 'Fake MMR'
--         Offset: 

-- The bit fields in this register are dummies with respect to address decoding. They are used as bit slice defines within the VHDL top decoder section to pick the proper address signals from the top level address bus (or VirtualBus) For each I/O unit, a MMR_CFG_ entry is defined. Later, these slice defines will be moved to the iomap definitions for the entire SoC (see also memmap.xml for memory map)
	constant R_decode_dummy      : regaddr_t := x"00";
	subtype               BV_MMR_CFG_fpga_registers is integer range 6 downto 0;
	subtype               BV_MMR_CFG_tap_registers is integer range 4 downto 0;

-------------------------------------------------------------------------
-- Address segment 'FPGA_Registers'
--         Offset: 0x000

	constant R_MagicId           : regaddr_t := x"00";
-- Control register
	constant R_Control           : regaddr_t := x"02";
-- Enable FPGA engine
	constant              B_ENABLE : natural := 0;
-- If set, slow down simulation
	constant              B_THROTTLE : natural := 1;
-- Virtual IRQ pin, H active
	constant              B_IRQ : natural := 6;
-- RESET FPGA engine
	constant              B_RESET : natural := 7;

-------------------------------------------------------------------------
-- Address segment 'TAP'
--         Offset: VBUS_ADDR_OFFSET

-- IDCODE register. Contains IDCODE of attached CPU.
	constant R_TAP_IDCODE        : regaddr_t := x"00";
-- EMUDATA register for data exchange between core/debug coprocessor and TAP
	constant R_TAP_EMUDATA       : regaddr_t := x"04";
-- Emulation control register. This is kept generic and CPU independent
	constant R_TAP_EMUCTRL       : regaddr_t := x"08";
-- Set this bit to request CPU to enter emulation
	constant              B_EMUREQ : natural := 0;
-- CPU specific select field for emudata input. Used on TAPs which require multiplexing of the EMUDATA register.
	subtype               BV_SELECT_REG is integer range 6 downto 4;
-- Some CPUs require the EMUEXEC bit to be toggled for executing an instruction. Refer to CPU debug reference section for details.
	constant              B_EMUEXEC : natural := 7;
-- Reset the CPU core.
	constant              B_CORE_RESET : natural := 15;
-- Emulation status register. The LSB contains the TAP generic status bis, MSB contains CPU specific bits. Refer to the CPU debug register description for detailed information.
	constant R_TAP_EMUSTAT       : regaddr_t := x"0a";
-- Emulation acknowledge. When an emureq request has been issued, this bit turns 1 when it is acknowledged by the attached CPU. The EMUACK bit remains high on most implementations when the CPU is in emulation mode.
	constant              B_EMUACK : natural := 0;
-- Signal that emulation is ready. Poll this bit to see if emulation is busy.
	constant              B_EMURDY : natural := 1;
-- Core specific status bits. See CPU debug reference.
	subtype               BV_CORE_SPEC is integer range 15 downto 8;
-- Emulation instruction register, 32 bit wide. This is used to feed instructions to the CPU core when in emulation mode.
	constant R_TAP_EMUIR         : regaddr_t := x"0c";
-- Program counter of attached CPU. Only valid when in emulation. Might be unused on some CPU cores.
	constant R_TAP_EMUPC         : regaddr_t := x"10";
-- Simulation only control register for Virtual TAP.
	constant R_TAP_SIMCTRL       : regaddr_t := x"18";
-- Throttle simulation if this bit is set and when no TAP access occurs
	constant              B_SIM_THROTTLE : natural := 0;
-- When Throttle bit set, simulation sleeps <this register's value>
	constant R_SIM_SLEEPCYCLES   : regaddr_t := x"19";


	-- Access records:

	type fpga_registers_ReadPort is record
		--! Exported value for register 'R_MagicId'
		magicid : std_logic_vector(REG_SIZE);
	end record;

	type fpga_registers_WritePort is record
		--! Exported value for register 'R_Control'
		--! Exported value for bit (vector) 'ENABLE'
		enable : std_logic;
		--! Exported value for bit (vector) 'THROTTLE'
		throttle : std_logic;
		--! Exported value for bit (vector) 'IRQ'
		irq : std_logic;
		--! Exported value for bit (vector) 'RESET'
		reset : std_logic;
	end record;

	type tap_registers_ReadPort is record
		--! Exported value for register 'R_TAP_IDCODE'
		tap_idcode : std_logic_vector(REG_SIZE4B);
		--! Exported value for register 'R_TAP_EMUDATA'
		tap_emudata : std_logic_vector(REG_SIZE4B);
		--! Exported value for register 'R_TAP_EMUSTAT'
		--! Exported value for bit (vector) 'EMUACK'
		emuack : std_logic;
		--! Exported value for bit (vector) 'EMURDY'
		emurdy : std_logic;
		--! Exported value for bit (vector) 'CORE_SPEC'
		core_spec : std_logic_vector(BV_CORE_SPEC);
		--! Exported value for register 'R_TAP_EMUPC'
		tap_emupc : std_logic_vector(REG_SIZE4B);
	end record;

	type tap_registers_WritePort is record
		--! Exported value for register 'R_TAP_EMUCTRL'
		--! Exported value for bit (vector) 'EMUREQ'
		emureq : std_logic;
		--! Exported value for bit (vector) 'SELECT_REG'
		select_reg : std_logic_vector(BV_SELECT_REG);
		--! Exported value for bit (vector) 'EMUEXEC'
		emuexec : std_logic;
		--! Exported value for bit (vector) 'CORE_RESET'
		core_reset : std_logic;
		--! Exported value for register 'R_TAP_EMUIR'
		tap_emuir : std_logic_vector(REG_SIZE4B);
		--! Exported value for register 'R_TAP_SIMCTRL'
		--! Exported value for bit (vector) 'SIM_THROTTLE'
		sim_throttle : std_logic;
		--! Exported value for register 'R_SIM_SLEEPCYCLES'
		sim_sleepcycles : std_logic_vector(REG_SIZE2B);
	end record;

	-- Decoder components:


---------------------------------------------------------
-- Decoder unit for 'FPGA_Registers'

component decode_fpga_registers is
	port (
		ce        : in  std_logic;
		ctrl      : out fpga_registers_WritePort;
		stat      : in  fpga_registers_ReadPort;
		data_in   : in  std_logic_vector(32-1 downto 0);
		data_out  : out std_logic_vector(32-1 downto 0);
		addr      : in  std_logic_vector(BV_MMR_CFG_fpga_registers);
		we        : in  std_logic;
		clk       : in  std_logic
	);
end component decode_fpga_registers;


---------------------------------------------------------
-- Decoder unit for 'TAP'

component decode_tap_registers is
	port (
		ce        : in  std_logic;
		ctrl      : out tap_registers_WritePort;
		stat      : in  tap_registers_ReadPort;
		data_in   : in  std_logic_vector(32-1 downto 0);
		data_out  : out std_logic_vector(32-1 downto 0);
		addr      : in  std_logic_vector(BV_MMR_CFG_tap_registers);
		we        : in  std_logic;
		clk       : in  std_logic
	);
end component decode_tap_registers;


end ghdlsim;


